@@ -5,7 +5,7 @@
             Name: __about__.py
             Description: information about this package
             Author: GentleCP
-            Email: 574881148@qq.com
+            Email: me@gentlecp.com
             WebSite: https://blog.gentlecp.com
             Create Date: 2021/4/18 
 -----------------End-----------------------------
@@ -14,7 +14,7 @@
 __name__ = 'corpwechatbot'
 __description__ = 'python interface for corpwechat message sending'
 __url__ = 'https://github.com/GentleCP/corpwechat-bot'
-__version__= '0.5.0'
+__version__= '0.6.0'
 __author__ = 'GentleCP'
 __author_email__ = 'me@gentlecp.com'
 __license__ = 'GPL v3'
\ No newline at end of file
